/*
 * passwd.c
 *  compatibility stuff for gcc-mingw32
 *
 *  Created by Dr A Olowofoyeku <laa12@keele.ac.uk>
 *    http://ourworld.compuserve.com/homepages/African_Chief/
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warrenties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * $Revision: 1.0 $
 * $Author: AAO $
 * $Date: 2000/02/03 $
 *
 */

#include <windows.h>
#include <lm.h>
#include <stdio.h>
#include <stdlib.h>

char *wide2ansi (LPWSTR wide)
{
char fcc[260];
   fcc[0] = NULL;
   WideCharToMultiByte ( CP_ACP, 0, wide, -1, fcc, 260, NULL, NULL );
   return &fcc;
}

struct passwd {
	char    *pw_name;       /* user name */
	char    *pw_passwd;     /* user password */
	int     pw_uid;         /* user id */
	int     pw_gid;         /* group id */
	char    *pw_gecos;      /* real name */
	char    *pw_dir;        /* home directory */
	char    *pw_shell;      /* shell program */
};

struct passwd *getpwnam(const char * name);
struct passwd *getpwnam(const char * name)
{
 WCHAR wname[256];
 struct _USER_INFO_3 *info;
 struct passwd psw;

    ZeroMemory (&psw, sizeof (psw));
    MultiByteToWideChar ( CP_ACP, 0, name,
        strlen (name)+1, wname, sizeof (wname) );

    if ( NetUserGetInfo ( NULL,
        (LPWSTR) &wname, 3, (LPBYTE *) &info ) )
      return NULL;

    psw.pw_name = wide2ansi(info->usri3_name);  	/* user name */
    psw.pw_gecos = wide2ansi(info->usri3_full_name);    /* real name */
    psw.pw_dir = wide2ansi(info->usri3_home_dir);       /* home directory */
    psw.pw_passwd = "***";     				/* user password */
    psw.pw_uid = info->usri3_user_id;          	 	/* user id */
    psw.pw_gid = info->usri3_primary_group_id;        	/* group id */

    NetApiBufferFree (info);
    return &psw;
}

struct passwd *getpwuid(int uid);
struct passwd *getpwuid(int uid)
{
 struct _USER_INFO_3 *info;
 struct passwd psw;
 DWORD count, entries, resume, i;

    ZeroMemory (&psw, sizeof (psw));
    resume = 0;
    
    NetUserEnum ( NULL, 3, FILTER_NORMAL_ACCOUNT, (LPBYTE *) &info,
                sizeof (*info), &count, &entries, &resume );

    for (i=0; i<count; i++)
    {
     if (info->usri3_user_id == uid)
     {
        psw.pw_name = wide2ansi(info->usri3_name);  		/* user name */       
    	psw.pw_gecos = wide2ansi(info->usri3_full_name);    	/* real name */
    	psw.pw_dir = wide2ansi(info->usri3_home_dir);       	/* home directory */
    	psw.pw_passwd = "***";     				/* user password */
    	psw.pw_uid = info->usri3_user_id;          	 	/* user id */
    	psw.pw_gid = info->usri3_primary_group_id;        	/* group id */
    	NetApiBufferFree (info);
    	return &psw;
     }
     info++;
    }
    NetApiBufferFree (info);
    return NULL;
}

/**********************************************************/
 /* use 2 globals */
 struct _USER_INFO_3 *info2 = NULL;
 int uid2 = -1;
 DWORD Counter = 0;

void setpwent(void);
void setpwent(void)
{
  int i;
  if (uid2 >= 0)
  {
    for (i = uid2; i >= 0; i--)
    {
      info2--;
    }
    uid2 = 0;
    Counter = 0;
  }
}

void endpwent(void);
void endpwent(void)
{
   NetApiBufferFree (info2);
   uid2 = -1;
   Counter = 0;
}

struct passwd *getpwent(void);
struct passwd *getpwent(void)
{
 struct passwd psw1;
 DWORD entries, resume;

  ZeroMemory (&psw1, sizeof (psw1)); /* fill with nulls */
  if (uid2 < 0)
  {
    uid2++;
    resume = 0;
    Counter = 0;
    if ( NetUserEnum ( NULL, 3, FILTER_NORMAL_ACCOUNT, (LPBYTE *) &info2,
                sizeof (*info2), &Counter, &entries, &resume ) )
        return NULL;
  }
  else
  { /* move to next record */
    uid2++;
    if (uid2 < Counter)
      info2++;
    else
      return NULL;  
  }

  psw1.pw_name = wide2ansi(info2->usri3_name); 		/* user name */
  psw1.pw_gecos = wide2ansi(info2->usri3_full_name);     /* real name */
  psw1.pw_dir = wide2ansi(info2->usri3_home_dir);        /* home directory */
  psw1.pw_passwd = "***";     				/* user password */
  psw1.pw_uid = info2->usri3_user_id;          	 	/* user id */
  psw1.pw_gid = info2->usri3_primary_group_id;        	/* group id */

  return &psw1;
}

/* trying to implement getpwuid() in another way */
struct passwd *getpwuid2(int uid)
{
 struct _USER_INFO_3 *info;
 DWORD count, entries, resume, i;
 struct passwd *psw0;
  
  psw0 = malloc (sizeof (*psw0));
  uid2 = -1;
  psw0 = getpwent();
  if (psw0 == NULL)
  {
    return NULL;
  }
  
  do
   {
     if (psw0->pw_uid == uid)
      {
        endpwent ();
        return psw0;
      }
     else
      {
        psw0 = getpwent();
        if (psw0 == NULL)
        {
  	  endpwent ();
  	  free (psw0);
  	  return NULL;
        }
      }
   } while (psw0 != NULL);

}

/* test to make sure things are running correctly */
/* must compile with -lnetapi32 to link the Netxxx functions */

int main()
{
struct passwd *psw2;
   psw2 = getpwuid (1000);
   if (psw2 != NULL)
   {
       printf ("ID = %i\n", psw2->pw_uid);
       printf ("Name = %s\n", psw2->pw_name);
       printf ("RealName = %s\n", psw2->pw_gecos);
       printf ("PassWord = %s\n", psw2->pw_passwd);
   }
   return 0;
}
